;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xd110029d01c4b7c7f392bce671a1673f . "./code.cg2js/cg-status-bar.fasl")
(#x6931e9bc08774199cc8fe7e23f05e5b3 . "./code.cg2js/cg-hotspot.fasl")
(#x45b9b1b0d878d5fb224faa5bf50c2f91 . "./code.cg2js/cg-toolbar.fasl")
(#x0ae136a058228c186922230b480528e7 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xdc66b5d96ebf767d2afdcf96eeff56f0 . "./code.cg2js/cg-tooltip.fasl")
(#x85f802f779ebea8c3695a8cc58d54270 . "./code.cg2js/cg-calendar.fasl")
(#xcb74d4adb4e8030a03bd5f93612ac2d3 . "./code.cg2js/cg-item-list.fasl")
(#x522a1410dfeee624509d143f45f83924 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xd0eff42dbb8083743316a10fc185896e . "./code.cg2js/cg-trackbar.fasl")
(#xac7739648f7e7123f62c093a8c38ac0e . "./code.cg2js/cg-class-grid.fasl")
(#x5c32639e791297012a62ce8c323e772f . "./code.cg2js/cg-dialog-item.fasl")
(#xe66b61d9c13ac15eeb27e899eea07b9a . "./code.cg2js/cg-comtab.fasl")
(#x17961192f8423105ca0b9e551a58eb5a . "./code.cg2js/cg-lisp-text.fasl")
(#x6a8e5128f3bca127a7557163470b3f76 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x0874791efa5957a56ecd3718caab2d53 . "./code.cg2js/cg-button.fasl")
(#x3a046729f945b3716a78b7dd9440cb35 . "./code.cg2js/cg-printing.fasl")
(#x139f6131aa73ac5f0167cabdd84bdad5 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xbab969e4bfbeff02b5aecbf054d31733 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x9a4c3d6a5d01039949f29323a1851d95 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x5303480e3cfa5d0dc334af36a6dfeca8 . "./code.cg2js/cg-selected-object.fasl")
(#x11155bd28515fe70e1a1cc9204696dac . "./code.cg2js/cg-text-or-combo.fasl")
(#x1082af0cead8d3e3dc2a31df53627991 . "./code.cg2js/cg-graphics-context.fasl")
(#xea7fb431a718c5cac1fae8de7a4cb7fb . "./code.cg2js/cg-edit-in-place.fasl")
(#xc70e745943939d33c609bcef39bb8e60 . "./code.cg2js/cg-string-dialog.fasl")
(#x0a848db93fd49a6a6afae51afb2850bc . "./code.cg2js/cg-list-view.fasl")
(#x0fa3a492f849709df39e76b997ed7303 . "./code.cg2js/cg-intersect.fasl")
(#x484f1a0d3a5a1ad47e450491e6821071 . "./code.cg2js/cg-text-widget.fasl")
(#xb82de6e4558e0a7876bbe76295668961 . "./code.cg2js/cg-directory-dialog.fasl")
(#x2628ef35eaf890dcb2a77cfbb9f5d795 . "./code.cg2js/cg-file-dialog.fasl")
(#xee7ead726bd7dce49779bbdec0aefc9b . "./code.cg2js/cg-dde-utils.fasl")
(#x0547eb4a35cee3909ef31fef6571dcca . "./code.cg2js/cg-nodes.fasl")
(#x7301112a3bf3b368e36f63acafa48ad3 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xc2c9d18bb6d7c0d8f721c7393d09649e . "./code.cg2js/cg-up-down-control.fasl")
(#xcde51903b1c34f53d5f08d198790c6cf . "./code.cg2js/cg-get-position.fasl")
(#xfc860cc2fd6c5ba86856453c40a3ff00 . "./code.cg2js/cg-toggling-widget.fasl")
(#x107b97a458e6e18c745d25c7eb175634 . "./code.cg2js/cg-acache.fasl")
(#xd042083991de74429e2e84f9b9209e56 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x2ee80af7cda17a0f977cfa4e97a5e02f . "./code.cg2js/cg-base.fasl")
(#x168285ce2a1e82e17be5d8fec47f3154 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x88447a8d398fab6ad287c6aab343fb8b . "./code.cg2js/cg-find-string-dialog.fasl")
(#x1e12aceca29e57de494aef924cd76cf5 . "./code.cg2js/cg-property.fasl")
(#x7076bb7488bbe36f2256d216f3eb3523 . "./code.cg2js/cg-editable-text.fasl")
(#xa6acffb97c7041cb5cc0d2025fe41d7f . "./code.cg2js/cg-lamp.fasl")
(#xe151a4a28f63104f400c30e8a61f3737 . "./code.cg2js/cg-progress-indicator.fasl")
(#xf2c6642db075111a3f47b59fceaf43d1 . "./code.cg2js/cg-chart-widget.fasl")
(#x5b6803079476eb40368177b3a2da3e13 . "./code.cg2js/cg-common-control.fasl")
(#x9adff5e324351475fc3ab28e147fbadc . "./code.cg2js/cg-font-dialog.fasl")
(#x51200719f006530db8f6a5f4132234a1 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x79ad4e71eb0ae8a2e7f707b3505980dc . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xc1e94815c4ecaa4fed7cff4816dde1dc . "./code.cg2js/cg-os-window.fasl")
(#x7e258581a785893a3f9f81d1e4257ff7 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xb7dc33dd063c5e8ce73ced33b62c94fa . "./code.cg2js/cg-pixmap.fasl")
(#xc02f2d779da235122775c3925038b08f . "./code.cg2js/cg-chart-or-plot.fasl")
(#x188610f86b0bcdb4479f76181e5087a4 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x251ff84dd8f270acedba0f84c5d9f99a . "./code.cg2js/cg-scaling-stream.fasl")
(#xe37e9ebf9b1505bb7a2b2a9ebf2d25c0 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xf5571fb17d03bd65755c85ea636c5ec9 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x14b0e397a05a307ad82e6ea435db7b37 . "./code.cg2js/cg-timer.fasl")
(#x2f90967845743185811f1222976f2725 . "./code.cg2js/cg-scroll-bar.fasl")
(#x2659084ab6ba9a05be5cee3a77839860 . "./code.cg2js/cg-caret.fasl")
(#xdbc0627a27e98250187981a55b6497fb . "./code.cg2js/cg-template-string.fasl")
(#x7db4c03fd9f9e0cc567c197b6e2c4082 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x63212bd9b72c97e47b69e5565127c4bd . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xcbe99360eb18584a174794cb9db15ae1 . "./code.cg2js/cg-rich-edit.fasl")
(#x58f072fa7ad47f75528bb9b5b66a29fb . "./code.cg2js/cg-combo-box.fasl")
(#x68ed79596821126eef4d266e126ef0ae . "./code.cg2js/cg-text-edit-pane.fasl")
(#x62406320157968fd9a9472742dde64bc . "./code.cg2js/cg-multi-picture-button.fasl")
(#xbd03feff4a6352f4e0a583a682ac1e62 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x163f51c9a6ccc299374e4db46046db71 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x0521bc08d3a3f7f61d71010efe7112ee . "./code.cg2js/cg-outline.fasl")
(#xbbb542bc6dbe15c122bf45f296d04f72 . "./code.cg2js/cg-class-support.fasl")
(#x0fd3c5a6177c4b6860439e261bf95576 . "./code.cg2js/cg-lettered-menu.fasl")
(#x96ab283402a26eae83165fa0144999e7 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xd828f87fb1da4873b7eaa1c4577aee1e . "./code.cg2js/cg-fill-texture.fasl")
(#x2cd3a6b1bc342de919dc9c3a2eb213a8 . "./code.cg2js/cg-lisp-widget.fasl")
(#x8d2ad370736bc54dd616b96b758b10dd . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x7a7206f328c52a2067e086f76796841e . "./code.cg2js/cg-clipboard.fasl")
(#x468bca529df42ea65e1ffd5e7f279dee . "./code.cg2js/cg-wrap-string.fasl")
(#x69848482e411d8b6b313eaf9a66a4e6f . "./code.cg2js/cg-os-widget.fasl")
(#x8e8f96f83a20ae9d0381a3de26c6f3c8 . "./code.cg2js/cg-check-box.fasl")
(#x6f4930a522cf2650d550902db975e4fa . "./code.cg2js/cg-choice-list.fasl")
(#x8f5f49f8807f2df75a9dfaf4039a3b01 . "./code.cg2js/cg-layout.fasl")
(#xbb1a5788b47caad14550c0ec58918e95 . "./code.cg2js/cg-group-box.fasl")
(#xbc2c414797482836319fede45a930bb9 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x6f7fc93bc27b2785a05994f15c863096 . "./code.cg2js/cg-radio-button.fasl")
(#x078bae94c25a26cddccce177ab1b3322 . "./code.cg2js/cg-tab-control.fasl")
(#x9a29ccea869bd7e5b0205dee3535612d . "./code.cg2js/cg.fasl")
(#xd11db3e1982e23e37baa5e7d000102bb . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x4d5f4872a45bd83e00b0eb96382f3d29 . "./code.cg2js/cg-dropping-outline.fasl")
(#x3e824ed5dccec464af6993b5a709a33f . "./code.cg2js/cg-get-pixmap.fasl")
(#xbfbcf69001e5110776dd46013ef1dd0a . "./code.cg2js/cg-grid-widget.fasl")
(#x6b32d62d499a7a10879c731de4059c0d . "./code.cg2js/cg-utility-dialog.fasl")
(#x10251ef6b2a3ea75690addec19fafd4d . "./code.cg2js/cg-static-text.fasl")
(#xdd0c8b1da0dfee8d9f627723561b9834 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xd2f8aabf86d26f9369eb92a8d42bfaba . "./code.cg2js/cg-object-editor-layout.fasl")
(#x0354763b6952a44fac3459c8faa89a49 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x7847d9a5de03aa3e3b94ecbd7ea484b5 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x7720b4ae06a11f7bd76ccc7ba11ac0e5 . "./code.cg2js/cg-plot-widget.fasl")
(#x66ef98406d38bc4e6f905675ec245f42 . "./code.cg2js/cg-color-dialog.fasl")
(#xb3c7fa45bf7ae9e82dce2a51e640b9af . "./code.cg2js/cg-icon-pixmap.fasl")
(#x0aacaa2ec71745333bb1986624a063e5 . "./code.cg2js/cg-icon.fasl")
(#xe199c32b84484ff43f5ac7b204412b42 . "./code.cg2js/cg-dev.fasl")
(#x0b572205878b60636b5bb7f31dbaa58d . "./code.cg2js/cg-choose-printer.fasl")
(#x3377da8b75f89df2799e813741d1afba . "./code.cg2js/cg-intersect-posbox.fasl")
(#xe7d1fe70cd4d1c43c29cdbbf65e73c36 . "./code.cg2js/cg-menu.fasl")
(#x2153718f0fe86072ee44c774a2b8a8d9 . "./code.cg2js/cg-yes-no-list.fasl")
(#x4c6ea99027cad8defdc83c63bc566198 . "./code.cg2js/cg-palette.fasl")
(#x39d5e46250c907801a0847b67e12e27c . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x32e3247a5470af51e77eac9340ec721a . "./code.cg2js/cg-message-dialog.fasl")
(#xd65da515ca08cbf60c008cf17883e10e . "./code.cg2js/cg-shortcut-menu.fasl")
(#x5e9b403eb52f2327ac7485d89679c58a . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xce4bd8af438b83c1fddad4f21fd10737 . "./code.cg2js/cg-split-bar.fasl")
(#xad5af57bf9a8d3e7ff96c6b56e681975 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x31fa62625f0e5c3b9509df5c3ab7ddd7 . "./code.cg2js/cg-web-browser.fasl")
(#xb9c1e8851e199f416a1df4fc2f4338cc . "./code.cg2js/cg-project-window.fasl")
(#x4de427789880869620fce3100f5de8ba . "./code.cg2js/cg-picture-widget.fasl")
(#x3cbfebd3d58f7412786928db02a44734 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xaec595b6628ea9b2c43294dc622c3772 . "./code.cg2js/cg-object-editor.fasl")
(#x3a73bb5e34b38bced3115a40d7eb2d73 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x836479deaa0559a5b8c8b9ddda67a3d1 . "./code.cg2js/cg-paren-matching.fasl")
(#xfa986242413088162c1cf6e2cc71975c . "./code.cg2js/cg-sample-file-menu.fasl")
(#xbf395e16510b3dbfb00de3afc57ccc0b . "./code.cg2js/cg-drawable.fasl")
(#xd3f37edaa1ac07ce40df0fbc1f1055ce . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x7d5e2616588aa975134351d2887a5fe3 . "./code.cg2js/cg-html-widget.fasl")
(#xb2bccebc3f92cc282a9fb80681dea20a . "./code.cg2js/cg-curve.fasl")
(#x8d5d6240d7be9222ec106a5ec4d9f6f5 . "./code.cg2js/cg-yes-no-string.fasl")
(#xc8623c105a379977a979abc0c522a9e0 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x88627d41397bc62309aaa5c8b73a18cc . "./code.cg2js/cg-html-dialog.fasl")
(#xa0af6354bb2cd8588f5ebcf205ac2f99 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x4b4991003bb83b2a776746f9f648db59 . "./code.cg2js/cg-tray.fasl")
(#x97e58d77df67a94b743b2f0b716812e7 . "./code.idejs/ide-apropos.fasl")
(#x6b9a16ad47221d2ab31d2b859c2da25c . "./code.idejs/ide-window-list.fasl")
(#x9d124c00e4c3459ec340d109df5d4162 . "./code.idejs/ide-class-browser.fasl")
(#x2e1becd38b510f182cf6a711b8fa901b . "./code.idejs/ide-base.fasl")
(#x756d2c148ff7d739443a763081fb5688 . "./code.idejs/ide-process-browser.fasl")
(#xb6142d6bc242d3b8e1b7e698a21c211a . "./code.idejs/ide-acache.fasl")
(#xaf09fe97fa1f4d45bc2a30ea0607e5f1 . "./code.idejs/ide-navigator.fasl")
(#xdb74430d5fc4a18aae53e0162a9aa315 . "./code.idejs/ide-trace.fasl")
(#x9b18d35c4d6d3a3000c1a4c752e694ee . "./code.idejs/ide-project.fasl")
(#x001120f6e416e2892ee8bc74a09aa589 . "./code.idejs/ide-patch-dialog.fasl")
(#x25ca66ec9e4a41e59f39c268f3ae0f11 . "./code.idejs/ide-grapher.fasl")
(#x05a2eb465857ebc5b8eee873cec5cba2 . "./code.idejs/ide-dev.fasl")
(#xc8a7bcca46c0ed4643bca8d8076fdb0f . "./code.idejs/ide-keyshort.fasl")
(#xc1efe877a732b9b9c9298ab39f3f2567 . "./code.idejs/ide-editor.fasl")
(#x4c02d33017e97ece5ab6cccacba95509 . "./code.idejs/ide-inspector.fasl")
(#xe161f739327548cb189e74d2eba4ddd8 . "./code.idejs/ide.fasl")
(#xac66d6535a72bf3e8938fe9593382caa . "./code.idejs/ide-clipboard.fasl")
(#x4f069f5b95449dec9b03eb8637cdc5c8 . "./code.idejs/ide-find-definitions.fasl")
(#x6e0b7098cb5d00919ec371ceff9ae7be . "./code.idejs/ide-form.fasl")
(#xd8674ce9420f22118213025d02616b69 . "./code.idejs/ide-intro.fasl")
(#x8f86e2c647e8ee9680cf23c9a2f43ddb . "./code.idejs/ide-find-in-files.fasl")
(#x48cea7546d8b68512181217171df3021 . "./code.idejs/ide-package-list.fasl")
(#xc86a5ee41fcb35dd0e5db207ce7ce4e9 . "./code.idejs/ide-class-grapher.fasl")
(#x696b64509ebe0538a3b58ac1081d4fa0 . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
